/* Copyright (C) 2017 Mono Wireless Inc. All Rights Reserved.    *
 * Released under MW-SLA-*J,*E (MONO WIRELESS SOFTWARE LICENSE   *
 * AGREEMENT).                                                   */

/*
 * シリアル書式(ASCII, BINARY)解釈用のプログラム。
 *
 * 本プログラムでは、標準入力から１バイトずつ入力を読み取り
 * ASCII書式またはBINARY書式を解釈します。
 * また解釈したデータ列をそのまま同じ書式で出力します。
 * 
 * マイコンなどの組込の参考にして下さい。
 * 
 * オプション:
 *  -b    BINARY書式
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*
 * シリアルパーサーの API
 */
#include "sercmd_gen.h"
#include "jendefs.h"
// システムのmsタイマー。タイムアウトを管理しないなら 0 のままとする。
uint32 u32TickCount_ms = 0;
// vOutput() で書式出力するときの１バイト出力用の関数
void vPutChar_STDOUT(uint8 u8char) {
	putchar(u8char);
}

/*
 * シリアルコマンドの解釈構造体
 */
tsSerCmd_Context serCmd;

/*
 * バッファの初期化
 */
void initSerCmd(int fBinary) {
	static uint8 u8buf[255]; // 入力系列を保持するバッファ
	if (fBinary)
		SerCmdBinary_vInit(&serCmd, u8buf, sizeof(u8buf));
	else
		SerCmdAscii_vInit(&serCmd, u8buf, sizeof(u8buf));

	serCmd.vPutChar = vPutChar_STDOUT; // 出力用の関数を登録
}

/* main */
int main(int argc, char *argv[]) {
	char keyinput[255]; /* バッファ */

	// -b でバイナリ書式の解釈
	int bBinary = 0;
	if (argc >= 2 && !strcmp(argv[1], "-b")) bBinary = 1;

	// 系列の初期化
	initSerCmd(bBinary);

	while (1) {
		/* コマンドの解釈は１バイトずつ行うが標準入力の fgets は１行ごとの入力
		 * であるため、解釈を初期化し行末までの入力とする
		 */
		int c;
		c = getchar();
		if(c < 0) break;

		uint8 u8stat;
		// １バイトずつ解釈をしていく
		u8stat = serCmd.u8Parse(&serCmd, c);

		// 解釈に成功したら終了
		if (u8stat == E_SERCMD_COMPLETE) {
			// 系列を読み出せたので表示
			printf("u8Parse() success, bytes=%d, payload=", serCmd.u16len); // データの長さは u16len
			int j;
			for(j = 0; j < serCmd.u16len; j++) {
				printf("%02x", serCmd.au8data[j]);  // ペイロードは au8data
			}
			printf("\n");

			// 入力した系列をそのまま vOutput で出力する
			//   出力先は vPutChar コマンドにて
			printf("vOutput() -> ");
			serCmd.vOutput(&serCmd);
			printf("\n");
		} else if (u8stat == E_SERCMD_CHECKSUM_ERROR) {
			// 系列を読み出せたので表示
			printf("Command checksum error: should be %x?\n", serCmd.u16cksum);
		}
	}
}

